unit WebModuleUnit1;

interface

uses
  ModelUnit, IObj,
  System.SysUtils, System.Classes, Web.HTTPApp, Datasnap.DSHTTPCommon,
  Datasnap.DSHTTPWebBroker, Datasnap.DSServer,
  Datasnap.DSAuth, IPPeerServer, Datasnap.DSCommonServer, Datasnap.DSHTTP;

type
  TWebModule1 = class(TWebModule)
    DSHTTPWebDispatcher1: TDSHTTPWebDispatcher;
    DSServer1: TDSServer;
    procedure WebModule1DefaultHandlerAction(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
  private
    function requestContainsPathInfo(Request: TWebRequest; const pathInfo : string) : boolean;
    procedure processItemsRequest(Request: TWebRequest; Response: TWebResponse);
    procedure processItemRequest(Request: TWebRequest; Response: TWebResponse);

    function getItemList: string;
  public
    class var FItemList : IIItemList;
    class procedure resetData;
    class procedure loadData(const data: string);
    class procedure updateItem(const code: string; clickCount: Integer);
  end;

var
  WebModuleClass: TComponentClass = TWebModule1;

implementation

{%CLASSGROUP 'System.Classes.TPersistent'}

{$R *.dfm}

uses Web.WebReq;

function TWebModule1.getItemList: string;
begin
  result := '';
  for var item in FItemList.Obj do
  begin
    if result <> '' then
      result := result + '#13#10';

    result := result + item.Obj.Name + ',' + item.Obj.Code + ',' + item.Obj.ClickCount.ToString;
  end;
end;

class procedure TWebModule1.loadData(const data : string);
begin
  FItemList := TObj<TIItemList>.CreateInstance(TIItemList.Create);

  var lst := TObj<TStringList>.CreateInstance(TStringList.Create);
  lst.Obj.Text := data;

  FItemList.Obj.clear;
  var lstItem := TObj<TStringList>.CreateInstance(TStringList.Create);
  for var i := 0 to lst.Obj.Count-1 do
  begin
    lstItem.Obj.Text := StringReplace(lst.Obj[i], ',', #13#10, [rfReplaceAll]);

    var name := trim(lstItem.Obj[0]);
    var code := trim(lstItem.Obj[1]);

    var clickCount := 0;
    if lstItem.Obj.count > 2 then
      clickCount := StrToIntDef(lstItem.Obj[2], 0);

    if (name <> '') and (code <> '') then
      FItemList.Obj.add(TObj<TItem>.CreateInstance(TItem.Create(code, name, clickCount)));
  end;
end;

procedure TWebModule1.processItemRequest(Request: TWebRequest; Response: TWebResponse);
begin
  var code := trim(Request.QueryFields.Values['code']);
  var clickCount := trim(Request.QueryFields.Values['clickCount']);

  TWebModule1.updateItem(code, StrToIntDef(clickCount, 0));
end;

procedure TWebModule1.processItemsRequest(Request: TWebRequest;
  Response: TWebResponse);
begin
  if Request.MethodType = mtGet then
    Response.Content := getItemList();
end;

function TWebModule1.requestContainsPathInfo(Request: TWebRequest;
  const pathInfo: string): boolean;
begin
  result := sametext(Request.PathInfo, pathInfo) or sametext(Request.PathInfo + '/', pathInfo);
end;

class procedure TWebModule1.resetData;
begin
  TWebModule1.loadData(C_COUNTRIES);
end;

class procedure TWebModule1.updateItem(const code: string; clickCount: Integer);
begin
  for var item in FItemList.Obj do
  begin
    if sameText(item.Obj.code, code) then
      item.Obj.ClickCount := clickCount;
  end;
end;

procedure TWebModule1.WebModule1DefaultHandlerAction(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
begin
  if requestContainsPathInfo(Request, '/items/') then
    processItemsRequest(Request, Response)

  else if requestContainsPathInfo(Request, '/item') then
    processItemRequest(Request, Response)

  else
    Response.Content :=
      '<html>' +
      '<head><title>DataSnap Server</title></head>' +
      '<body>DataSnap Server</body>' +
      '</html>';
end;

initialization

finalization
  Web.WebReq.FreeWebModules;

end.

