unit AutoHeightFlowLayout;

interface
uses
  FMX.Layouts, FMX.Controls, System.Classes, System.SysUtils;

type
  // Add autoHeight property
  TAutoHeightFlowLayout = class(TFlowLayout)
  private
    FAutoHeight : boolean;
    procedure SetAutoHeight(const Value: boolean);
    procedure AdjustHeight();
  protected
    procedure DoRealign; override;
  published
    property AutoHeight : boolean read FAutoHeight write SetAutoHeight;
  public
    constructor Create(AOwner: TComponent); override;
  end;

implementation

{ TMyFlowLayout }

procedure TAutoHeightFlowLayout.AdjustHeight;
begin
  var H := double(0);

  for var c in self.Controls do
  begin
    if H < (c.Position.y + c.Height) then
      H := c.Position.Y + c.Height + VerticalGap + c.Margins.Bottom + self.Padding.Bottom;
  end;

  Height := H;
end;

constructor TAutoHeightFlowLayout.Create(AOwner: TComponent);
begin
  inherited;
  FAutoHeight := True;
end;

procedure TAutoHeightFlowLayout.DoRealign;
begin
  inherited;

  if FAutoHeight then
    AdjustHeight();
end;

procedure TAutoHeightFlowLayout.SetAutoHeight(const Value: boolean);
begin
  FAutoHeight := Value;
end;

end.
