unit FrameItemDetails;

interface

uses
  ModelUnit,

  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Graphics, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.StdCtrls,
  FMX.Controls.Presentation, FMX.Edit, FMX.Objects, FMX.Ani;

type
  TfrItemDetails = class(TFrame)
    lblItem: TLabel;
    background: TRectangle;
    procedure FrameClick(Sender: TObject);
    procedure FrameMouseEnter(Sender: TObject);
    procedure FrameMouseLeave(Sender: TObject);
  private
    FWidth, FHeight, FFontSize : Double;
    FItem : TItem;

    FOnItemMouseLeave: TNotifyEvent;
    FOnItemClick: TNotifyEvent;
    FOnItemMouseEnter: TNotifyEvent;

    procedure SetOnItemClick(const Value: TNotifyEvent);
    procedure SetOnItemMouseEnter(const Value: TNotifyEvent);
    procedure SetOnItemMouseLeave(const Value: TNotifyEvent);
  public
    property Item : TItem read FItem;

    property OnItemClick : TNotifyEvent read FOnItemClick write SetOnItemClick;
    property OnItemMouseEnter : TNotifyEvent read FOnItemMouseEnter write SetOnItemMouseEnter;
    property OnItemMouseLeave : TNotifyEvent read FOnItemMouseLeave write SetOnItemMouseLeave;

    procedure bindData(item: TItem);
    procedure refreshData;

    procedure setScales(const scaleX, scaleY : Double);
    procedure setBGColor(const color : TAlphaColor);
    constructor Create(AOwner : TComponent); override;
  end;

  TItemView = TfrItemDetails;

implementation

{$R *.fmx}

{ TfrItemDetails }

procedure TfrItemDetails.bindData(item: TItem);
begin
  FItem := item;
  lblItem.Text := item.Description;
end;

constructor TfrItemDetails.Create(AOwner: TComponent);
begin
  inherited;
  FWidth := Width;
  FHeight := Height;
  FFontSize := lblItem.Font.Size;
end;

procedure TfrItemDetails.FrameClick(Sender: TObject);
begin
  if assigned(FOnItemClick) then
    FOnItemClick(Sender);
end;

procedure TfrItemDetails.FrameMouseEnter(Sender: TObject);
begin
  if assigned(FOnItemMouseEnter) then
    FOnItemMouseEnter(Sender);
end;

procedure TfrItemDetails.FrameMouseLeave(Sender: TObject);
begin
  if assigned(FOnItemMouseLeave) then
    FOnItemMouseLeave(Sender);
end;

procedure TfrItemDetails.refreshData;
begin
  bindData(FItem);
end;

procedure TfrItemDetails.setBGColor(const color: TAlphaColor);
begin
  background.Fill.Color := color;
end;

procedure TfrItemDetails.SetOnItemClick(const Value: TNotifyEvent);
begin
  FOnItemClick := Value;
end;

procedure TfrItemDetails.SetOnItemMouseEnter(const Value: TNotifyEvent);
begin
  FOnItemMouseEnter := Value;
end;

procedure TfrItemDetails.SetOnItemMouseLeave(const Value: TNotifyEvent);
begin
  FOnItemMouseLeave := Value;
end;

procedure TfrItemDetails.setScales(const scaleX, scaleY: Double);
begin
  Width := round(scaleX * FWidth);
  Height := round(scaleY * FHeight);
  lblItem.Font.Size := round(FFontSize * scaleX);
end;

end.
