unit FrameItemList;

interface

uses
  AutoHeightFlowLayout, ModelUnit, Generics.Collections, MainDM,
  FrameItemDetails, System.Threading,

  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Graphics, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.StdCtrls,
  FMX.Layouts, FMX.Ani;

type
  TItemViewList = TList<TItemView>;

  TfrItemList = class(TFrame)
    scrollboxItems: TVertScrollBox;
  private
    FLayoutItems : TAutoHeightFlowLayout;
    FItemList : TItemList;
    FItemViews : TItemViewList;

    procedure createItemViews;
    procedure createLayoutItems;

    procedure ItemViewMouseEnter(Sender: TObject);
    procedure ItemViewMouseLeave(Sender: TObject);

    procedure ItemViewClick(Sender: TObject);
    procedure ItemViewClick1(Sender: TObject);
    procedure ItemViewClick2(Sender: TObject);

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure bindData(itemList : TItemList);
    procedure refreshData;

    procedure clearItemViews;
  end;

  TItemListView = TfrItemList;

implementation

{$R *.fmx}

{ TfrItemList }

procedure TfrItemList.bindData(itemList: TItemList);
begin
  FLayoutItems.BeginUpdate;

  if FItemList <> itemList then
  begin
    FItemList.Free;
    FItemList := ItemList;
  end;
  clearItemViews;

  for var Item in FItemList do
  begin
    var ItemView := TItemView.Create(nil);
    ItemView.Name := 'Item' + Item.Code;
    ItemView.bindData(Item);

    itemView.Margins.Top := 10;
    itemView.Margins.Bottom := 10;
    itemView.Margins.Left := 10;
    itemView.Margins.Right := 10;

    ItemView.parent := FLayoutItems;
    FItemViews.Add(ItemView);

    ItemView.OnItemMouseEnter := ItemViewMouseEnter;
    ItemView.OnItemMouseLeave := ItemViewMouseLeave;
    ItemView.OnItemClick := ItemViewClick;

    ItemViewMouseLeave(ItemView);
  end;

  FLayoutItems.EndUpdate;
end;

procedure TfrItemList.refreshData;
begin
  bindData(FItemList);
end;

procedure TfrItemList.clearItemViews;
begin
  for var i := FItemViews.Count - 1 downto 0 do
  begin
    FItemViews[i].Parent := nil;
    FItemViews[i].Free;
  end;

  FItemViews.Clear;
end;


constructor TfrItemList.Create(AOwner: TComponent);
begin
  inherited;
  createLayoutItems;
  createItemViews;
end;

procedure TfrItemList.createItemViews;
begin
  FItemViews := TItemViewList.Create;
end;

procedure TfrItemList.createLayoutItems;
begin
  FLayoutItems := TAutoHeightFlowLayout.Create(self);
  FLayoutItems.Parent := scrollboxItems;
  FLayoutItems.Align := TAlignLayout.Top;
  FLayoutItems.Height := FLayoutItems.Height;

  FLayoutItems.Padding.Left := 20;
  FLayoutItems.Padding.Right := 20;
  FLayoutItems.Padding.Top := 20;
  FLayoutItems.Padding.Bottom := 20;
end;

destructor TfrItemList.Destroy;
begin
  FItemList.free;
  FItemViews.Free;

  inherited;
end;

procedure TfrItemList.ItemViewClick(Sender: TObject);
begin
  ItemViewClick1(Sender);
  ItemViewClick2(Sender);
end;

procedure TfrItemList.ItemViewClick1(Sender: TObject);
begin
  ShowMessage('Itemview clicked ' + TItemView(Sender).Item.Code);
end;

procedure TfrItemList.ItemViewClick2(Sender: TObject);
begin
  TItemView(Sender).Item.IncrementClickCount;
  TItemView(Sender).refreshData;
  dmMain.refreshData;
end;

procedure TfrItemList.ItemViewMouseEnter(Sender: TObject);
begin
  FLayoutItems.BeginUpdate;
  if Sender is TItemView then
  begin
    TItemView(Sender).setBGColor(TAlphaColors.Lightblue);
    TItemView(Sender).setScales(1.1, 1.1);
  end;
  FLayoutItems.EndUpdate;
end;

procedure TfrItemList.ItemViewMouseLeave(Sender: TObject);
begin
  FLayoutItems.BeginUpdate;
  if Sender is TItemView then
  begin
    TItemView(Sender).setBGColor(TAlphaColors.LtGray);
    TItemView(Sender).setScales(1, 1);
  end;
  FLayoutItems.EndUpdate;
end;

end.
