unit MainDM;

interface

uses
  ModelUnit,
  System.SysUtils, System.Classes, Generics.Collections;

type
  TdmMain = class(TDataModule)
    procedure DataModuleCreate(Sender: TObject);
    procedure DataModuleDestroy(Sender: TObject);
  private
    FItemList : TItemObjectList;
    FOnLoadData: TProc;
    procedure LoadData(const data: string); overload;
    procedure SetOnLoadData(const Value: TProc);
  public
    property OnLoadData : TProc read FOnLoadData write SetOnLoadData;
    function GetItemList(filter : TPredicate<TItem>) : TItemList;
    procedure LoadData; overload;
    procedure refreshData;
  end;

var
  dmMain: TdmMain;

implementation

{%CLASSGROUP 'FMX.Controls.TControl'}

{$R *.dfm}

{ TdmMain }

procedure TdmMain.DataModuleCreate(Sender: TObject);
begin
  FItemList := TItemObjectList.Create;
end;

procedure TdmMain.DataModuleDestroy(Sender: TObject);
begin
  FItemList.Free;
end;

function TdmMain.GetItemList(filter: TPredicate<TItem>): TItemList;
begin
  Result := TItemList.Create;

  for var item in FItemList do
  begin
    if assigned(filter) then
    begin
      if filter(item) then
        Result.Add(item);
    end
    else
      Result.Add(item);
  end;
end;

procedure TdmMain.LoadData;
begin
  LoadData(C_COUNTRIES);
  refreshData;
end;

procedure TdmMain.refreshData;
begin
  if assigned(FOnLoadData) then
    FOnLoadData();
end;

procedure TdmMain.SetOnLoadData(const Value: TProc);
begin
  FOnLoadData := Value;
end;

procedure TdmMain.LoadData(const data: string);
begin
  var lst := TStringList.Create;
  try
    lst.Text := StringReplace(data, '#13#10', #13#10, [rfReplaceAll]);

    FItemList.clear;
    var lstItem := TStringList.Create;
    try
      for var i := 0 to lst.Count-1 do
      begin
        lstItem.Text := StringReplace(lst[i], ',', #13#10, [rfReplaceAll]);

        var name := trim(lstItem[0]);
        var code := trim(lstItem[1]);

        var clickCount := 0;
        if lstItem.count > 2 then
          clickCount := StrToIntDef(lstItem[2], 0);

        if (name <> '') and (code <> '') then
          FItemList.add(TItem.Create(code, name, clickCount));
      end;
    finally
      lstItem.Free;
    end;

  finally
    lst.Free;
  end;
end;
end.
