unit MainForm;

interface

uses
  FrameItemDetails, FrameItemList, MainDM, ModelUnit,
  Generics.Collections, Generics.Defaults,

  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs,
  FMX.Controls.Presentation, FMX.StdCtrls, FMX.Layouts, FMX.Objects,
  FMX.TabControl, FMX.Ani;

type
  TfrmMain = class(TForm)
    tcPages: TTabControl;
    tbHome: TTabItem;
    tbItemList: TTabItem;
    tbSelectedItemList: TTabItem;
    tbOrderedItemList: TTabItem;
    btnLoadData: TButton;
    btnShowData: TButton;
    btnClear: TButton;
    tmrDelayedShowData: TTimer;
    Label1: TLabel;
    procedure btnLoadDataClick(Sender: TObject);
    procedure btnShowDataClick(Sender: TObject);
    procedure btnClearClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure tmrDelayedShowDataTimer(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    FItemListView: TItemListView;
    FSelectedItemListView: TItemListView;
    FOrderedItemListView : TItemListView;

    procedure createItemListView;
    procedure createSelectedItemListView;
    procedure createOrderedItemListView;

    procedure showData;
    procedure subscribeOnDataChange;
  public
  end;

var
  frmMain : TfrmMain;

implementation

{$R *.fmx}

procedure TfrmMain.btnClearClick(Sender: TObject);
begin
  FItemListView.clearItemViews;
  FSelectedItemListView.clearItemViews;
  FOrderedItemListView.clearItemViews;
end;

procedure TfrmMain.btnLoadDataClick(Sender: TObject);
begin
  dmMain.loadData;
end;


procedure TfrmMain.btnShowDataClick(Sender: TObject);
begin
  showData;
end;

procedure TfrmMain.createItemListView;
begin
  FItemListView := TItemListView.Create(self);
  FItemListView.Name := 'ItemListView';

  FItemListView.AddFreeNotify(tbItemList);

  FItemListView.align := TAlignLayout.Client;
  FItemListView.Parent := tbItemList;
end;

procedure TfrmMain.createSelectedItemListView;
begin
  FSelectedItemListView := TItemListView.Create(self);
  FSelectedItemListView.Name := 'SelectedItemListView';

  FSelectedItemListView.AddFreeNotify(tbSelectedItemList);

  FSelectedItemListView.align := TAlignLayout.Client;
  FSelectedItemListView.Parent := tbSelectedItemList;
end;

procedure TfrmMain.createOrderedItemListView;
begin
  FOrderedItemListView := TItemListView.Create(self);
  FOrderedItemListView.Name := 'OrderedItemListView';

  FOrderedItemListView.AddFreeNotify(tbOrderedItemList);

  FOrderedItemListView.align := TAlignLayout.Client;
  FOrderedItemListView.Parent := tbOrderedItemList;
end;


procedure TfrmMain.FormCreate(Sender: TObject);
begin
  tcPages.ActiveTab := tbHome;

  createItemListView;
  createSelectedItemListView;
  createOrderedItemListView;
end;

procedure TfrmMain.FormShow(Sender: TObject);
begin
  subscribeOnDataChange;
  dmMain.loadData;
end;

procedure TfrmMain.showData;
const
  MaxItemCount = 0;
begin
  begin
    var count := 0;
    FItemListView.bindData(dmMain.GetItemList(
        function (Item : TItem) : boolean
        begin
          inc(count);
          result := (MaxItemCount = 0) or (count < MaxItemCount);
        end)
    );
  end;

  begin
    var count := 0;
    FSelectedItemListView.bindData(dmMain.GetItemList(
        function (Item : TItem) : boolean
        begin
          if Item.ClickCount > 0 then
          begin
            inc(count);
            result := (MaxItemCount = 0) or (count < MaxItemCount);
          end
          else
            result := false;
        end)
    );
  end;

  begin
    var count := 0;
    var itemList := dmMain.GetItemList(
        function (Item : TItem) : boolean
        begin
          inc(count);
          result := (MaxItemCount = 0) or (count < MaxItemCount);
        end);

    itemList.Sort (
      TComparer<TItem>.Construct(
        function (const item1, item2 : TItem) : Integer
        begin
          var c := item2.ClickCount - item1.ClickCount;

         if c = 0 then
            result := CompareText(item1.Code, item2.Code)
          else
          result := c;
        end
      )
    );

    FOrderedItemListView.bindData(itemList);
  end;

end;

procedure TfrmMain.subscribeOnDataChange;
begin
  dmMain.OnLoadData := procedure
                        begin
                          // I need to use the timer to help avoiding access violation
                          // due to UI controls being referenced and deleted and recreate
                          // at the same time.
                          tmrDelayedShowData.Enabled := true;
                        end;
end;

procedure TfrmMain.tmrDelayedShowDataTimer(Sender: TObject);
begin
  tmrDelayedShowData.Enabled := false;
  showData;
end;

end.
