unit FrameItemList;

interface

uses
  AutoHeightFlowLayout, ModelUnit, Generics.Collections, IObj, MainDM,
  FrameItemDetails,

  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Graphics, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.StdCtrls,
  FMX.Layouts, FMX.Ani;

type
  IItemView = IObj<TItemView>;
  TIItemViewList = TList<IItemView>;
  IIItemViewList = IObj<TIItemViewList>;

  TfrItemList = class(TFrame)
    scrollboxItems: TVertScrollBox;
  private
    FLayoutItems : TAutoHeightFlowLayout;
    FItemList : IItemList;
    FItemViews : IIItemViewList;

    procedure createItemViews;
    procedure createLayoutItems;

    procedure ItemViewMouseEnter(Sender: TObject);
    procedure ItemViewMouseLeave(Sender: TObject);

    procedure ItemViewClick(Sender: TObject);
    procedure ItemViewClick1(Sender: TObject);
    procedure ItemViewClick2(Sender: TObject);
  public
    constructor Create(AOwner: TComponent); override;

    procedure bindData(itemList : IItemList);
    procedure refreshData;

    procedure clearItemViews;
  end;

  TItemListView = TfrItemList;
  IItemListView = IObj<TItemListView>;

implementation

{$R *.fmx}

{ TfrItemList }

procedure TfrItemList.bindData(itemList: IItemList);
begin
  FLayoutItems.BeginUpdate;

  FItemList := ItemList;
  clearItemViews;

  for var Item in FItemList.Obj do
  begin
    var itemView := TObj<TItemView>.CreateInstance(TItemView.Create(nil));
    ItemView.Obj.Name := 'Item' + Item.Code;
    ItemView.Obj.bindData(Item);

    itemView.Obj.Margins.Top := 10;
    itemView.Obj.Margins.Bottom := 10;
    itemView.Obj.Margins.Left := 10;
    itemView.Obj.Margins.Right := 10;

    ItemView.Obj.parent := FLayoutItems;
    FItemViews.Obj.Add(ItemView);

    // Note
    ItemView.Obj.OnItemMouseEnter := ItemViewMouseEnter;
    ItemView.Obj.OnItemMouseLeave := ItemViewMouseLeave;
    ItemView.Obj.OnItemClick := ItemViewClick;

    ItemViewMouseLeave(ItemView.Obj);
  end;

  FLayoutItems.EndUpdate;
end;

procedure TfrItemList.refreshData;
begin
  bindData(FItemList);
end;

procedure TfrItemList.clearItemViews;
begin
  FItemViews.Obj.Clear;
end;


constructor TfrItemList.Create(AOwner: TComponent);
begin
  inherited;
  createLayoutItems;
  createItemViews;
end;

procedure TfrItemList.createItemViews;
begin
  FItemViews := TObj<TIItemViewList>.CreateInstance(TIItemViewList.Create);
end;

procedure TfrItemList.createLayoutItems;
begin
  FLayoutItems := TAutoHeightFlowLayout.Create(self);
  FLayoutItems.Parent := scrollboxItems;
  FLayoutItems.Align := TAlignLayout.Top;
  FLayoutItems.Height := FLayoutItems.Height;

  FLayoutItems.Padding.Left := 20;
  FLayoutItems.Padding.Right := 20;
  FLayoutItems.Padding.Top := 20;
  FLayoutItems.Padding.Bottom := 20;
end;

procedure TfrItemList.ItemViewClick(Sender: TObject);
begin
  ItemViewClick1(Sender);
  ItemViewClick2(Sender);
end;

procedure TfrItemList.ItemViewClick1(Sender: TObject);
begin
  ShowMessage('Itemview clicked ' + TItemView(Sender).Item.Code);
end;

procedure TfrItemList.ItemViewClick2(Sender: TObject);
begin
  TItemView(Sender).Item.IncrementClickCount;
  TItemView(Sender).refreshData;
  dmMain.refreshData;
end;

procedure TfrItemList.ItemViewMouseEnter(Sender: TObject);
begin
  FLayoutItems.BeginUpdate;
  if Sender is TItemView then
  begin
    TItemView(Sender).setBGColor(TAlphaColors.Lightblue);
    TItemView(Sender).setScales(1.1, 1.1);
  end;
  FLayoutItems.EndUpdate;
end;

procedure TfrItemList.ItemViewMouseLeave(Sender: TObject);
begin
  FLayoutItems.BeginUpdate;
  if Sender is TItemView then
  begin
    TItemView(Sender).setBGColor(TAlphaColors.LtGray);
    TItemView(Sender).setScales(1, 1);
  end;
  FLayoutItems.EndUpdate;
end;

end.
