unit MainDM;

interface

uses
  ModelUnit, IObj, EventHandlers,
  System.SysUtils, System.Classes, Generics.Collections;

type
  TdmMain = class(TDataModule)
    procedure DataModuleCreate(Sender: TObject);
  private
    FItemList : IIItemList;
    FOnLoadData : IObj<TProcEvents>;
    procedure LoadData(const data: string); overload;

    procedure createItemList;
    procedure createOnLoadData;
    function GetOnLoadData: TProcEvents;
  public
    property OnLoadData : TProcEvents read GetOnLoadData;
    function GetItemList(filter : TPredicate<TItem>) : IItemList;
    procedure LoadData; overload;
    procedure refreshData;
  end;

var
  dmMain: TdmMain;

implementation

{%CLASSGROUP 'FMX.Controls.TControl'}

{$R *.dfm}

{ TdmMain }

procedure TdmMain.DataModuleCreate(Sender: TObject);
begin
  createItemList;
  createOnLoadData;
end;

procedure TdmMain.createItemList;
begin
  FItemList := TObj<TIItemList>.CreateInstance(TIItemList.Create);
end;

procedure TdmMain.createOnLoadData;
begin
  FOnLoadData := TObj<TProcEvents>.CreateInstance(TProcEvents.Create(nil));
end;

function TdmMain.GetItemList(filter: TPredicate<TItem>): IItemList;
begin
  Result := TObj<TItemList>.CreateInstance(TItemList.Create);

  for var item in FItemList.Obj do
  begin
    if assigned(filter) then
    begin
      if filter(item.Obj) then
        Result.Obj.Add(item.Obj);
    end
    else
      Result.Obj.Add(item.Obj);
  end;
end;

function TdmMain.GetOnLoadData: TProcEvents;
begin
  result := FOnLoadData.Obj;
end;

procedure TdmMain.LoadData;
begin
  LoadData(C_COUNTRIES);
  refreshData;
end;

procedure TdmMain.refreshData;
begin
  FOnLoadData.Obj.execute;
end;

procedure TdmMain.LoadData(const data: string);
begin
  var lst := TObj<TStringList>.CreateInstance(TStringList.Create);
  lst.Obj.Text := StringReplace(data, '#13#10', #13#10, [rfReplaceAll]);

  FItemList.Obj.clear;
  var lstItem := TObj<TStringList>.CreateInstance(TStringList.Create);
  for var i := 0 to lst.Obj.Count-1 do
  begin
    lstItem.Obj.Text := StringReplace(lst.Obj[i], ',', #13#10, [rfReplaceAll]);

    var name := trim(lstItem.Obj[0]);
    var code := trim(lstItem.Obj[1]);

    var clickCount := 0;
    if lstItem.Obj.count > 2 then
      clickCount := StrToIntDef(lstItem.Obj[2], 0);

    if (name <> '') and (code <> '') then
      FItemList.Obj.add(TObj<TItem>.CreateInstance(TItem.Create(code, name, clickCount)));
  end;
end;
end.
