unit FrameItemDetails;

interface

uses
  ModelUnit, IObj, EventHandlers,

  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Graphics, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.StdCtrls,
  FMX.Controls.Presentation, FMX.Edit, FMX.Objects, FMX.Ani;

type
  TfrItemDetails = class(TFrame)
    lblItem: TLabel;
    background: TRectangle;
    procedure FrameClick(Sender: TObject);
    procedure FrameMouseEnter(Sender: TObject);
    procedure FrameMouseLeave(Sender: TObject);
  private
    FWidth, FHeight, FFontSize : Double;
    FItem : TItem;

    FOnItemMouseLeave: IObj<TMethodEvents>;
    FOnItemClick: IObj<TMethodEvents>;
    FOnItemMouseEnter: IObj<TMethodEvents>;

    procedure createOnItemClick;
    procedure createOnItemMouseEnter;
    procedure createOnItemMouseLeave;

    function getOnItemClick: TMethodEvents;
    function getOnItemMouseEnter: TMethodEvents;
    function getOnItemMouseLeave: TMethodEvents;
  public
    property Item : TItem read FItem;

    property OnItemClick : TMethodEvents read getOnItemClick;
    property OnItemMouseEnter : TMethodEvents read getOnItemMouseEnter;
    property OnItemMouseLeave : TMethodEvents read getOnItemMouseLeave;

    procedure bindData(item: TItem);
    procedure refreshData;

    procedure setScales(const scaleX, scaleY : Double);
    procedure setBGColor(const color : TAlphaColor);
    constructor Create(AOwner : TComponent); override;
  end;

  TItemView = TfrItemDetails;
  IItemView = IObj<TItemView>;

implementation

{$R *.fmx}

{ TfrItemDetails }

procedure TfrItemDetails.bindData(item: TItem);
begin
  FItem := item;
  lblItem.Text := item.Description;
end;

constructor TfrItemDetails.Create(AOwner: TComponent);
begin
  inherited;
  FWidth := Width;
  FHeight := Height;
  FFontSize := lblItem.Font.Size;

  createOnItemClick;
  createOnItemMouseEnter;
  createOnItemMouseLeave;
end;

procedure TfrItemDetails.createOnItemClick;
begin
  FOnItemClick := TObj<TMethodEvents>.CreateInstance(TMethodEvents.Create(nil));
end;

procedure TfrItemDetails.createOnItemMouseEnter;
begin
  FOnItemMouseEnter := TObj<TMethodEvents>.CreateInstance(TMethodEvents.Create(nil));
end;

procedure TfrItemDetails.createOnItemMouseLeave;
begin
  FOnItemMouseLeave := TObj<TMethodEvents>.CreateInstance(TMethodEvents.Create(nil));
end;

procedure TfrItemDetails.FrameClick(Sender: TObject);
begin
  FOnItemClick.Obj.execute(Sender);
end;

procedure TfrItemDetails.FrameMouseEnter(Sender: TObject);
begin
  FOnItemMouseEnter.Obj.execute(Sender);
end;

procedure TfrItemDetails.FrameMouseLeave(Sender: TObject);
begin
  FOnItemMouseLeave.Obj.execute(Sender);
end;

function TfrItemDetails.getOnItemClick: TMethodEvents;
begin
  result := FOnItemClick.Obj;
end;

function TfrItemDetails.getOnItemMouseEnter: TMethodEvents;
begin
  result := FOnItemMouseEnter.Obj;
end;

function TfrItemDetails.getOnItemMouseLeave: TMethodEvents;
begin
  result := FOnItemMouseLeave.Obj;
end;

procedure TfrItemDetails.refreshData;
begin
  bindData(FItem);
end;

procedure TfrItemDetails.setBGColor(const color: TAlphaColor);
begin
  background.Fill.Color := color;
end;

procedure TfrItemDetails.setScales(const scaleX, scaleY: Double);
begin
  Width := round(scaleX * FWidth);
  Height := round(scaleY * FHeight);
  lblItem.Font.Size := round(FFontSize * scaleX);
end;

end.
