unit MainDM;

interface

uses
  ModelUnit, IObj, RestMessageCenter, EventHandlers,
  System.SysUtils, System.Classes, Generics.Collections, Datasnap.DSClientRest,
  Data.Bind.Components, Data.Bind.ObjectScope, REST.Client,
  System.Net.URLClient, System.Net.HttpClient, System.Net.HttpClientComponent,
  IdBaseComponent, IdComponent, IdTCPConnection, IdTCPClient, IdHTTP;

const
  baseURL = 'http://localhost:8080';
  protocol = 'http';
  host = 'localhost';
  port = 8080;

type
  TdmMain = class(TDataModule)
    procedure DataModuleCreate(Sender: TObject);
  private
    FItemList : IIItemList;
    FDSRestConnection: IObj<TDSRestConnection>;
    FRestMessageCenter: IObj<TRestMessageCenter>;
    FOnLoadData : IObj<TProcEvents>;

    procedure createServerConnection();
    procedure connectToServer();
    procedure createRestMessageCenter();

    function GetURLAsString(const aURL: string): string;

    procedure createItemList;
    procedure createOnLoadData;
    function GetOnLoadData: TProcEvents;
    procedure LoadData(const data: string); overload;
  public
    property OnLoadData : TProcEvents read GetOnLoadData;
    function GetItemList(filter : TPredicate<TItem>) : IItemList;
    procedure LoadData; overload;
    procedure refreshData;
    procedure UpdateItem(code : string; clickCount: Integer);
  end;

var
  dmMain: TdmMain;

implementation

{%CLASSGROUP 'FMX.Controls.TControl'}

{$R *.dfm}

{ TdmMain }

procedure TdmMain.connectToServer();
begin
  if not assigned(FDSRestConnection) then
    createServerConnection;

  self.FDSRestConnection.Obj.Protocol := protocol;
  self.FDSRestConnection.Obj.Host := host;
  self.FDSRestConnection.Obj.Port := port;
end;

procedure TdmMain.createItemList;
begin
  FItemList := TObj<TIItemList>.CreateInstance(TIItemList.Create);
end;

procedure TdmMain.createOnLoadData;
begin
  FOnLoadData := TObj<TProcEvents>.CreateInstance(TProcEvents.Create(nil));
end;

procedure TdmMain.createRestMessageCenter;
begin
  FRestMessageCenter := TObj<TRestMessageCenter>.CreateInstance(TRestMessageCenter.Create(nil));
  FRestMessageCenter.Obj.Connection := self.FDSRestConnection.Obj;

  FRestMessageCenter.Obj.CreateChannel('RefreshData');
  FRestMessageCenter.Obj.OnMessage['RefreshData'].addHandler(
    procedure (s: string)
    begin
      LoadData;
    end
  );
end;

procedure TdmMain.createServerConnection;
begin
  FDSRestConnection := TObj<TDSRestConnection>.CreateInstance(TDSRestConnection.Create(nil));
  FDSRestConnection.Obj.LoginPrompt := false;
end;

procedure TdmMain.DataModuleCreate(Sender: TObject);
begin
  createItemList;
  createServerConnection;
  connectToServer;
  createOnLoadData;
  createRestMessageCenter;
end;

function TdmMain.GetItemList(filter: TPredicate<TItem>): IItemList;
begin
  Result := TObj<TItemList>.CreateInstance(TItemList.Create);

  for var item in FItemList.Obj do
  begin
    if assigned(filter) then
    begin
      if filter(item.Obj) then
        Result.Obj.Add(item.Obj);
    end
    else
      Result.Obj.Add(item.Obj);
  end;
end;

function TdmMain.GetOnLoadData: TProcEvents;
begin
  result := FOnLoadData.Obj;
end;

function TdmMain.GetURLAsString(const aURL: string): string;
begin
  var lHTTP := TObj<TIdHTTP>.CreateInstance(TIdHTTP.Create);
  Result := lHTTP.Obj.Get(aURL);
end;

procedure TdmMain.LoadData;
begin
  var data := GetURLAsString(Format('%s/items', [baseURL]));
  LoadData(data);
  refreshData;
end;

procedure TdmMain.refreshData;
begin
  FOnLoadData.Obj.execute;
end;

procedure TdmMain.LoadData(const data: string);
begin
  var lst := TObj<TStringList>.CreateInstance(TStringList.Create);
  lst.Obj.Text := StringReplace(data, '#13#10', #13#10, [rfReplaceAll]);

  FItemList.Obj.clear;
  var lstItem := TObj<TStringList>.CreateInstance(TStringList.Create);
  for var i := 0 to lst.Obj.Count-1 do
  begin
    lstItem.Obj.Text := StringReplace(lst.Obj[i], ',', #13#10, [rfReplaceAll]);

    var name := trim(lstItem.Obj[0]);
    var code := trim(lstItem.Obj[1]);

    var clickCount := 0;
    if lstItem.Obj.count > 2 then
      clickCount := StrToIntDef(lstItem.Obj[2], 0);

    if (name <> '') and (code <> '') then
      FItemList.Obj.add(TObj<TItem>.CreateInstance(TItem.Create(code, name, clickCount)));
  end;
end;

procedure TdmMain.UpdateItem(code: string; clickCount: Integer);
begin
  GetURLAsString(format('%s/item?code=%s&clickCount=%d', [baseUrl, code, clickCount]));
  self.FRestMessageCenter.Obj.broadCast('RefreshData', 'Update Item');
end;

end.
