unit MainForm;

interface

uses
  FrameItemDetails, FrameItemList, MainDM, ModelUnit,
  Generics.Collections, Generics.Defaults, IObj, System.Threading,

  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs,
  FMX.Controls.Presentation, FMX.StdCtrls, FMX.Layouts, FMX.Objects,
  FMX.TabControl, FMX.Ani, FMX.Memo.Types, FMX.ScrollBox, FMX.Memo, FMX.Edit;

type
  TfrmMain = class(TForm)
    tcPages: TTabControl;
    tbHome: TTabItem;
    tbItemList: TTabItem;
    tbSelectedItemList: TTabItem;
    tbOrderedItemList: TTabItem;
    btnLoadData: TButton;
    btnShowData: TButton;
    btnClear: TButton;
    tmrDelayedShowData: TTimer;
    Label2: TLabel;
    Label1: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    procedure btnLoadDataClick(Sender: TObject);
    procedure btnShowDataClick(Sender: TObject);
    procedure btnClearClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure tmrDelayedShowDataTimer(Sender: TObject);
  private
    FItemListView: IItemListView;
    FSelectedItemListView: IItemListView;
    FOrderedItemListView : IItemListView;

    procedure createItemListView;
    procedure createSelectedItemListView;
    procedure createOrderedItemListView;

    procedure showData;
    procedure subscribeOnDataChange;
  public
  end;

var
  frmMain : TfrmMain;

implementation

{$R *.fmx}

procedure TfrmMain.btnClearClick(Sender: TObject);
begin
  FItemListView.Obj.clearItemViews;
  FSelectedItemListView.Obj.clearItemViews;
  FOrderedItemListView.Obj.clearItemViews;
end;

procedure TfrmMain.btnLoadDataClick(Sender: TObject);
begin
  dmMain.loadData;
end;


procedure TfrmMain.btnShowDataClick(Sender: TObject);
begin
  showData;
end;

procedure TfrmMain.createItemListView;
begin
  FItemListView := TObj<TItemListView>.CreateInstance(TItemListView.Create(nil));
  FItemListView.Obj.Name := 'ItemListView';

  FItemListView.Obj.align := TAlignLayout.Client;
  FItemListView.Obj.Parent := tbItemList;
end;

procedure TfrmMain.createSelectedItemListView;
begin
  FSelectedItemListView := TObj<TItemListView>.CreateInstance(TItemListView.Create(nil));
  FSelectedItemListView.Obj.Name := 'SelectedItemListView';

  FSelectedItemListView.Obj.align := TAlignLayout.Client;
  FSelectedItemListView.Obj.Parent := tbSelectedItemList;
end;

procedure TfrmMain.createOrderedItemListView;
begin
  FOrderedItemListView := TObj<TItemListView>.CreateInstance(TItemListView.Create(nil));
  FOrderedItemListView.Obj.Name := 'OrderedItemListView';

  FOrderedItemListView.Obj.align := TAlignLayout.Client;
  FOrderedItemListView.Obj.Parent := tbOrderedItemList;
end;


procedure TfrmMain.FormCreate(Sender: TObject);
begin
  tcPages.ActiveTab := tbHome;

  createItemListView;
  createSelectedItemListView;
  createOrderedItemListView;
end;

procedure TfrmMain.FormShow(Sender: TObject);
begin
  subscribeOnDataChange;
  dmMain.loadData;
end;

procedure TfrmMain.showData;
const
  MaxItemCount = 0;
begin
  begin
    var count := 0;
    FItemListView.Obj.bindData(dmMain.GetItemList(
        function (Item : TItem) : boolean
        begin
          inc(count);
          result := (MaxItemCount = 0) or (count < MaxItemCount);
        end)
    );
  end;

  begin
    var count := 0;
    FSelectedItemListView.Obj.bindData(dmMain.GetItemList(
        function (Item : TItem) : boolean
        begin
          if Item.ClickCount > 0 then
          begin
            inc(count);
            result := (MaxItemCount = 0) or (count < MaxItemCount);
          end
          else
            result := false;
        end)
    );
  end;

  begin
    var count := 0;
    var itemList := dmMain.GetItemList(
        function (Item : TItem) : boolean
        begin
          inc(count);
          result := (MaxItemCount = 0) or (count < MaxItemCount);
        end);

    itemList.Obj.Sort (
      TComparer<TItem>.Construct(
        function (const item1, item2 : TItem) : Integer
        begin
          var c := item2.ClickCount - item1.ClickCount;

          if c = 0 then
            result := CompareText(item1.Code, item2.Code)
          else
            result := c;
        end
      )
    );

    FOrderedItemListView.Obj.bindData(itemList);
  end;

end;

procedure TfrmMain.subscribeOnDataChange;
begin
  dmMain.OnLoadData.subscribe(self,
    procedure
    begin
      // I need to use the timer to help avoiding access violation
      // due to UI controls being referenced and deleted and recreate
      // at the same time.
      tmrDelayedShowData.Enabled := true;
    end
  );
end;

procedure TfrmMain.tmrDelayedShowDataTimer(Sender: TObject);
begin
  tmrDelayedShowData.Enabled := false;
  showData;
end;

end.
