unit IObj;

interface

uses
  System.Classes, System.SysUtils;

type
  {
    // usage
    var myObj := TObj<TMyObject>.CreateInstance(TMyObject.Create..);

    // do something with myObj.Obj
    myObj.Obj
  }

  TBaseObj<T : class> = class(TComponent)
  private
    FObj: T;
    procedure SetObj(const Value: T);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    destructor Destroy; override;
  end;

  IObj<T> = Interface(IInterface)
    ['{C71075DF-1BCE-4A7E-935B-643DE4C785D6}']
    function GetObj: T;
    property Obj: T read GetObj;

  end;

  TObj<T: class> = class(TInterfacedObject, IObj<T>)
  private
    FObj : T;
    FBaseObj : TBaseObj<T>;
    function GetObj: T;

  public
    constructor Create(Obj: T); reintroduce;
    destructor Destroy; override;

    class function CreateInstance(Obj: T) : IObj<T>;
  end;


implementation

constructor TObj<T>.Create(Obj: T);
begin
  inherited Create;

  if Obj is TComponent then
  begin
    FBaseObj := TBaseObj<T>.Create(nil);
    FBaseObj.SetObj(Obj);
  end
  else
    FObj := Obj;
end;

class function TObj<T>.CreateInstance(Obj: T): IObj<T>;
begin
  result := TObj<T>.Create(Obj);
end;

destructor TObj<T>.Destroy;
begin
  FBaseObj.Free;
  FObj.free;
  inherited;
end;

function TObj<T>.GetObj: T;
begin
  if assigned(FBaseObj) then
    Result := FBaseObj.FObj
  else
    Result := FObj;
end;

{ TBaseObj<T> }

destructor TBaseObj<T>.Destroy;
begin
  FObj.free;
  inherited;
end;

procedure TBaseObj<T>.Notification(AComponent: TComponent; Operation: TOperation);
begin
  if assigned(AComponent) and (Operation = TOperation.opRemove) then
    FObj := nil;

  inherited;
end;

procedure TBaseObj<T>.SetObj(const Value: T);
begin
  FObj := Value;

  if FObj is TComponent then
    TComponent(FObj).FreeNotification(self);
end;

end.
