unit ModelUnit;

interface
uses
  SysUtils, Generics.Collections, IObj;

type
  TItem = class
  private
    FName: string;
    FCode: string;
    FClickCount: Integer;
    FDescription: string;

    procedure SetCode(const Value: string);
    procedure SetName(const Value: string);
    procedure SetClickCount(const Value: Integer);
    procedure UpdateDescription;
  public
    property Code : string read FCode write SetCode;
    property Name : string read FName write SetName;
    property ClickCount : Integer read FClickCount write SetClickCount;
    property Description : string read FDescription;
    procedure IncrementClickCount;

    constructor Create(const Code, Name : string; const clickCount : Integer = 0);
  end;

  TItemObjectList = TObjectList<TItem>;
  TItemList = TList<TItem>;

  // Use IObj
  IItemList = IObj<TItemList>;
  IItem = IObj<TItem>;
  TIItemList = TList<IItem>;
  IIItemList = IObj<TIItemList>;

const
  C_COUNTRIES =
      'Afghanistan,  AF'#13#10+
      'land Islands, AX'#13#10+
      'Albania,	AL'#13#10+
      'Algeria,	DZ'#13#10+
      'American Samoa,	AS'#13#10+
      'Andorra, AD'#13#10+
      'Angola,	AO'#13#10+
      'Anguilla,	AI'#13#10+
      'Antarctica,	AQ'#13#10+
      'Antigua and Barbuda,	AG'#13#10+
      'Argentina,	AR'#13#10+
      'Armenia,	AM'#13#10+
      'Aruba,	AW'#13#10+
      'Australia,	AU'#13#10+
      'Austria,	AT'#13#10+
      'Azerbaijan,	AZ'#13#10+
      'Bahrain,	BH'#13#10+
      'Bahamas,	BS'#13#10+
      'Bangladesh,	BD'#13#10+
      'Barbados,	BB'#13#10+
      'Belarus,	BY'#13#10+
      'Belgium,	BE'#13#10+
      'Belize,	BZ'#13#10+
      'Benin,	BJ'#13#10+
      'Bermuda,	BM'#13#10+
      'Bhutan,	BT'#13#10+
      'Bolivia,	BO'#13#10+
      'Bonaire, BQ'#13#10+
      'Bosnia and Herzegovina,	BA'#13#10+
      'Botswana,	BW'#13#10+
      'Bouvet Island,	BV'#13#10+
      'Brazil,	BR'#13#10+
      'British Indian Ocean Territory,	IO'#13#10+
      'Brunei Darussalam,	BN'#13#10+
      'Bulgaria,	BG'#13#10+
      'Burkina Faso,	BF'#13#10+
      'Burundi,	BI'#13#10+
      'Cambodia,	KH'#13#10+
      'Cameroon,	CM'#13#10+
      'Canada,	CA'#13#10+
      'Cape Verde,	CV'#13#10+
      'Cayman Islands,	KY'#13#10+
      'Central African Republic,	CF'#13#10+
      'Chad,	TD'#13#10+
      'Chile,	CL'#13#10+
      'China,	CN'#13#10+
      'Christmas Island,	CX'#13#10+
      'Cocos (Keeling) Islands,	CC'#13#10+
      'Colombia,	CO'#13#10+
      'Comoros,	KM'#13#10+
      'Congo,	CG'#13#10+
      'Congo, CD'#13#10+
      'Cook Islands,	CK'#13#10+
      'Costa Rica,	CR'#13#10+
      'Cte d''Ivoire,	CI'#13#10+
      'Croatia,	HR'#13#10+
      'Cuba,	CU'#13#10+
      'Curaao,	CW'#13#10+
      'Cyprus,	CY'#13#10+
      'Czech Republic,	CZ'#13#10+
      'Denmark,	DK'#13#10+
      'Djibouti,	DJ'#13#10+
      'Dominica,	DM'#13#10+
      'Dominican Republic,	DO'#13#10+
      'Ecuador,	EC'#13#10+
      'Egypt,	EG'#13#10+
      'El Salvador,	SV'#13#10+
      'Equatorial Guinea,	GQ'#13#10+
      'Eritrea,	ER'#13#10+
      'Estonia,	EE'#13#10+
      'Ethiopia,	ET'#13#10+
      'Falkland Islands (Malvinas),	FK'#13#10+
      'Faroe Islands,	FO'#13#10+
      'Fiji,	FJ'#13#10+
      'Finland,	FI'#13#10+
      'France,	FR'#13#10+
      'French Guiana,	GF'#13#10+
      'French Polynesia,	PF'#13#10+
      'French Southern Territories,	TF'#13#10+
      'Gabon,	GA'#13#10+
      'Gambia,	GM'#13#10+
      'Georgia,	GE'#13#10+
      'Germany,	DE'#13#10+
      'Ghana,	GH'#13#10+
      'Gibraltar,	GI'#13#10+
      'Greece,	GR'#13#10+
      'Greenland,	GL'#13#10+
      'Grenada,	GD'#13#10+
      'Guadeloupe,	GP'#13#10+
      'Guam,	GU'#13#10+
      'Guatemala,	GT'#13#10+
      'Guernsey,	GG'#13#10+
      'Guinea,	GN'#13#10+
      'Guinea-Bissau,	GW'#13#10+
      'Guyana,	GY'#13#10+
      'Haiti,	HT'#13#10+
      'Heard Island and McDonald Islands,	HM'#13#10+
      'Holy See (Vatican City State),	VA'#13#10+
      'Honduras,	HN'#13#10+
      'Hong Kong,	HK'#13#10+
      'Hungary,	HU'#13#10+
      'Iceland,	IS'#13#10+
      'India,	IN'#13#10+
      'Indonesia,	ID'#13#10+
      'Iran, IR'#13#10+
      'Iraq,	IQ'#13#10+
      'Ireland,	IE'#13#10+
      'Isle of Man,	IM'#13#10+
      'Israel,	IL'#13#10+
      'Italy,	IT'#13#10+
      'Jamaica,	JM'#13#10+
      'Japan,	JP'#13#10+
      'Jersey,	JE'#13#10+
      'Jordan,	JO'#13#10+
      'Kazakhstan,	KZ'#13#10+
      'Kenya,	KE'#13#10+
      'Kiribati,	KI'#13#10+
      'North Korea, KP'#13#10+
      'Korea, KR'#13#10+
      'Kuwait,	KW'#13#10+
      'Kyrgyzstan,	KG'#13#10+
      'Lao, LA'#13#10+
      'Latvia,	LV'#13#10+
      'Lebanon,	LB'#13#10+
      'Lesotho,	LS'#13#10+
      'Liberia,	LR'#13#10+
      'Libya,	LY'#13#10+
      'Liechtenstein,	LI'#13#10+
      'Lithuania,	LT'#13#10+
      'Luxembourg,	LU'#13#10+
      'Macao,	MO'#13#10+
      'Macedonia, MK'#13#10+
      'Madagascar,	MG'#13#10+
      'Malawi,	MW'#13#10+
      'Malaysia,	MY'#13#10+
      'Maldives,	MV'#13#10+
      'Mali,	ML'#13#10+
      'Malta,	MT'#13#10+
      'Marshall Islands,	MH'#13#10+
      'Martinique,	MQ'#13#10+
      'Mauritania,	MR'#13#10+
      'Mauritius,	MU'#13#10+
      'Mayotte,	YT'#13#10+
      'Mexico,	MX'#13#10+
      'Micronesia, FM'#13#10+
      'Moldova, MD'#13#10+
      'Monaco,	MC'#13#10+
      'Mongolia,	MN'#13#10+
      'Montenegro,	ME'#13#10+
      'Montserrat,	MS'#13#10+
      'Morocco,	MA'#13#10+
      'Mozambique,	MZ'#13#10+
      'Myanmar,	MM'#13#10+
      'Namibia,	NA'#13#10+
      'Nauru,	NR'#13#10+
      'Nepal,	NP'#13#10+
      'Netherlands,	NL'#13#10+
      'New Caledonia,	NC'#13#10+
      'New Zealand,	NZ'#13#10+
      'Nicaragua,	NI'#13#10+
      'Niger,	NE'#13#10+
      'Nigeria,	NG'#13#10+
      'Niue,	NU'#13#10+
      'Norfolk Island,	NF'#13#10+
      'Northern Mariana Islands,	MP'#13#10+
      'Norway,	NO'#13#10+
      'Oman,	OM'#13#10+
      'Pakistan,	PK'#13#10+
      'Palau,	PW'#13#10+
      'Palestine, PS'#13#10+
      'Panama,	PA'#13#10+
      'Papua New Guinea,	PG'#13#10+
      'Paraguay,	PY'#13#10+
      'Peru,	PE'#13#10+
      'Philippines,	PH'#13#10+
      'Pitcairn,	PN'#13#10+
      'Poland,	PL'#13#10+
      'Portugal,	PT'#13#10+
      'Puerto Rico,	PR'#13#10+
      'Qatar,	QA'#13#10+
      'Runion,	RE'#13#10+
      'Romania,	RO'#13#10+
      'Russian Federation,	RU'#13#10+
      'Rwanda,	RW'#13#10+
      'Saint Barthlemy,	BL'#13#10+
      'Saint Helena,	SH'#13#10+
      'Saint Kitts and Nevis,	KN'#13#10+
      'Saint Lucia,	LC'#13#10+
      'Saint Martin (French part),	MF'#13#10+
      'Saint Pierre and Miquelon,	PM'#13#10+
      'Saint Vincent and the Grenadines,	VC'#13#10+
      'Samoa,	WS'#13#10+
      'San Marino,	SM'#13#10+
      'Sao Tome and Principe,	ST'#13#10+
      'Saudi Arabia,	SA'#13#10+
      'Senegal,	SN'#13#10+
      'Serbia,	RS'#13#10+
      'Seychelles,	SC'#13#10+
      'Sierra Leone,	SL'#13#10+
      'Singapore,	SG'#13#10+
      'Sint Maarten (Dutch part),	SX'#13#10+
      'Slovakia,	SK'#13#10+
      'Slovenia,	SI'#13#10+
      'Solomon Islands,	SB'#13#10+
      'Somalia,	SO'#13#10+
      'South Africa,	ZA'#13#10+
      'South Georgia and the South Sandwich Islands,	GS'#13#10+
      'South Sudan,	SS'#13#10+
      'Spain,	ES'#13#10+
      'Sri Lanka,	LK'#13#10+
      'Sudan,	SD'#13#10+
      'Suriname,	SR'#13#10+
      'Svalbard and Jan Mayen,	SJ'#13#10+
      'Swaziland,	SZ'#13#10+
      'Sweden,	SE'#13#10+
      'Switzerland,	CH'#13#10+
      'Syrian Arab Republic,	SY'#13#10+
      'Taiwan, TW'#13#10+
      'Tajikistan,	TJ'#13#10+
      'Tanzania, TZ'#13#10+
      'Thailand,	TH'#13#10+
      'Timor-Leste,	TL'#13#10+
      'Togo,	TG'#13#10+
      'Tokelau,	TK'#13#10+
      'Tonga,	TO'#13#10+
      'Trinidad and Tobago,	TT'#13#10+
      'Tunisia,	TN'#13#10+
      'Turkey,	TR'#13#10+
      'Turkmenistan,	TM'#13#10+
      'Turks and Caicos Islands,	TC'#13#10+
      'Tuvalu,	TV'#13#10+
      'Uganda,	UG'#13#10+
      'Ukraine,	UA'#13#10+
      'United Arab Emirates,	AE'#13#10+
      'United Kingdom,	GB'#13#10+
      'United States,	US'#13#10+
      'United States Minor Outlying Islands,	UM'#13#10+
      'Uruguay,	UY'#13#10+
      'Uzbekistan,	UZ'#13#10+
      'Vanuatu,	VU'#13#10+
      'Venezuela, VE'#13#10+
      'Viet Nam,	VN'#13#10+
      'Virgin Islands, VG'#13#10+
      'Virgin Islands, VI'#13#10+
      'Wallis and Futuna,	WF'#13#10+
      'Western Sahara,	EH'#13#10+
      'Yemen,	YE'#13#10+
      'Zambia,	ZM'#13#10+
      'Zimbabwe,	ZW';



implementation

{ TItem }

constructor TItem.Create(const Code, Name: string; const clickCount : Integer);
begin
  FCode := Code;
  FName := Name;
  FClickCount := clickCount;

  UpdateDescription;
end;

procedure TItem.SetCode(const Value: string);
begin
  FCode := Value;
  UpdateDescription;
end;

procedure TItem.SetName(const Value: string);
begin
  FName := Value;
  updateDescription;
end;

procedure TItem.UpdateDescription;
begin
  FDescription := FCode + ' - ' + FName + ' (' + FClickCount.ToString + ')';
end;

procedure TItem.IncrementClickCount;
begin
  inc(FClickCount);
  UpdateDescription;
end;

procedure TItem.SetClickCount(const Value: Integer);
begin
  FClickCount := Value;
  UpdateDescription;
end;

end.
