unit FrameItemList;

interface

uses
  AutoHeightFlowLayout, ModelUnit, Generics.Collections, MainDM,
  FrameItemDetails,

  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.ExtCtrls;

type
  TItemViewList = TList<TItemView>;

  TfrItemList = class(TFrame)
  private
    FLayoutItems : TAutoHeightFlowLayout;
    FItemList : TItemList;
    FItemViews : TItemViewList;

    procedure createItemViews;
    procedure createLayoutItems;

    procedure ItemViewMouseEnter(Sender: TObject);
    procedure ItemViewMouseLeave(Sender: TObject);

    procedure ItemViewClick(Sender: TObject);
    procedure ItemViewClick1(Sender: TObject);
    procedure ItemViewClick2(Sender: TObject);

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure bindData(itemList : TItemList);
    procedure refreshData;

    procedure clearItemViews;
  end;

  TItemListView = TfrItemList;

implementation

{$R *.dfm}

{ TfrItemList }

procedure TfrItemList.bindData(itemList: TItemList);
begin
  FLayoutItems.LockDrawing;

  if FItemList <> itemList then
  begin
    FItemList.Free;
    FItemList := ItemList;
  end;
  clearItemViews;

  for var item in FItemList do
  begin
    var itemView := TItemView.Create(nil);
    itemView.Name := 'Item' + item.Code;
    itemView.bindData(item);

    itemView.Margins.Top := 10;
    itemView.Margins.Bottom := 10;
    itemView.Margins.Left := 10;
    itemView.Margins.Right := 10;
    itemView.AlignWithMargins := true;

    itemView.parent := FLayoutItems;
    FItemViews.Add(itemView);

    itemView.OnItemMouseEnter := ItemViewMouseEnter;
    itemView.OnItemMouseLeave := ItemViewMouseLeave;
    itemView.OnItemClick := ItemViewClick;

    ItemViewMouseLeave(ItemView);
  end;

  FLayoutItems.UnLockDrawing;
end;

procedure TfrItemList.refreshData;
begin
  bindData(FItemList);
end;

procedure TfrItemList.clearItemViews;
begin
  for var i := FItemViews.Count - 1 downto 0 do
  begin
    FItemViews[i].Parent := nil;
    FItemViews[i].Free;
  end;

  FItemViews.Clear;
end;

constructor TfrItemList.Create(AOwner: TComponent);
begin
  inherited;

  createLayoutItems;
  createItemViews;
end;

procedure TfrItemList.createItemViews;
begin
  FItemViews := TItemViewList.Create;
end;

procedure TfrItemList.createLayoutItems;
begin
  FLayoutItems := TAutoHeightFlowLayout.Create(self);
  FLayoutItems.Parent := self;
  FLayoutItems.Align := TAlign.alTop;
  FLayoutItems.Visible := true;
  FLayoutItems.Height := FLayoutItems.Height;

  FLayoutItems.Padding.Left := 20;
  FLayoutItems.Padding.Right := 20;
  FLayoutItems.Padding.Top := 20;
  FLayoutItems.Padding.Bottom := 20;

  FLayoutItems.BevelInner := bvNone;
  FLayoutItems.BevelOuter := bvNone;
end;

destructor TfrItemList.Destroy;
begin
  FItemList.Free;
  FItemViews.Free;

  inherited;
end;

procedure TfrItemList.ItemViewClick(Sender: TObject);
begin
  ItemViewClick1(Sender);
  ItemViewClick2(Sender);
end;

procedure TfrItemList.ItemViewClick1(Sender: TObject);
begin
  ShowMessage('Itemview clicked ' + TItemView(Sender).Item.Code);
end;

procedure TfrItemList.ItemViewClick2(Sender: TObject);
begin
  TItemView(Sender).Item.IncrementClickCount;
  TItemView(Sender).refreshData;
  dmMain.refreshData;
end;

procedure TfrItemList.ItemViewMouseEnter(Sender: TObject);
begin
  if Sender is TItemView then
  begin
    TItemView(Sender).setBGColor(clWebLightBlue);
    TItemView(Sender).setScales(1.1, 1.1);
  end;
end;

procedure TfrItemList.ItemViewMouseLeave(Sender: TObject);
begin
  if Sender is TItemView then
  begin
    TItemView(Sender).setBGColor(clLtGray);
    TItemView(Sender).setScales(1, 1);
  end;
end;

end.
