unit FrameItemDetails;

interface

uses
  ModelUnit, IObj,

  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls;

type
  TfrItemDetails = class(TFrame)
    lblItem: TLabel;

    procedure lblItemMouseEnter(Sender: TObject);
    procedure lblItemMouseLeave(Sender: TObject);

    procedure lblItemClick(Sender: TObject);

  private
    FWidth, FHeight, FFontSize : Double;
    FItem : TItem;

    FOnItemClick: TNotifyEvent;
    FOnItemMouseEnter: TNotifyEvent;
    FOnItemMouseLeave: TNotifyEvent;

    procedure SetOnItemClick(const Value: TNotifyEvent);
    procedure SetOnItemMouseEnter(const Value: TNotifyEvent);
    procedure SetOnItemMouseLeave(const Value: TNotifyEvent);
   public
    property Item : TItem read FItem;

    property OnItemClick : TNotifyEvent read FOnItemClick write SetOnItemClick;
    property OnItemMouseEnter : TNotifyEvent read FOnItemMouseEnter write SetOnItemMouseEnter;
    property OnItemMouseLeave : TNotifyEvent read FOnItemMouseLeave write SetOnItemMouseLeave;

    procedure bindData(item: TItem);
    procedure refreshData;

    procedure setScales(const scaleX, scaleY : Double);
    procedure setBGColor(const color : TColor);
    constructor Create(AOwner : TComponent); override;
  end;

  TItemView = TfrItemDetails;

implementation

{$R *.dfm}

{ TfrItemDetails }

procedure TfrItemDetails.bindData(item: TItem);
begin
  FItem := item;
  lblItem.Caption := item.Description;
end;

constructor TfrItemDetails.Create(AOwner: TComponent);
begin
  inherited;
  FWidth := width;
  FHeight := height;
  FFontSize := lblItem.Font.Size;
end;

procedure TfrItemDetails.lblItemClick(Sender: TObject);
begin
  if assigned(FOnItemClick) then
    FOnItemClick(self);
end;

procedure TfrItemDetails.lblItemMouseEnter(Sender: TObject);
begin
  if assigned(FOnItemMouseEnter) then
    FOnItemMouseEnter(self);
end;

procedure TfrItemDetails.lblItemMouseLeave(Sender: TObject);
begin
  if assigned(FOnItemMouseLeave) then
    FOnItemMouseLeave(self);
end;

procedure TfrItemDetails.refreshData;
begin
  bindData(FItem);
end;

procedure TfrItemDetails.setBGColor(const color: TColor);
begin
  lblItem.Color := color;
end;

procedure TfrItemDetails.SetOnItemClick(const Value: TNotifyEvent);
begin
  FOnItemClick := Value;
end;

procedure TfrItemDetails.SetOnItemMouseEnter(const Value: TNotifyEvent);
begin
  FOnItemMouseEnter := Value;
end;

procedure TfrItemDetails.SetOnItemMouseLeave(const Value: TNotifyEvent);
begin
  FOnItemMouseLeave := Value;
end;

procedure TfrItemDetails.setScales(const scaleX, scaleY: Double);
begin
  Width := round(scaleX * FWidth);
  Height := round(scaleY * FHeight);
  lblItem.Font.Size := round(FFontSize * scaleX);
end;

end.
