unit MainForm;

interface

uses
  FrameItemDetails, FrameItemList, MainDM, ModelUnit, IObj,
  Generics.Collections, Generics.Defaults,

  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants,
  System.Classes, Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs,
  Vcl.ComCtrls, Vcl.StdCtrls, Vcl.ExtCtrls;

type
  TfrmMain = class(TForm)
    pcPages: TPageControl;
    tbHome: TTabSheet;
    btnLoadData: TButton;
    btnShowData: TButton;
    btnClear: TButton;
    tbItemList: TTabSheet;
    tbSelectedItemList: TTabSheet;
    tbOrderedItemList: TTabSheet;
    tmrDelayedShowData: TTimer;
    Label1: TLabel;
    Label2: TLabel;
    procedure btnLoadDataClick(Sender: TObject);
    procedure btnShowDataClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnClearClick(Sender: TObject);
    procedure tmrDelayedShowDataTimer(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    FItemListView: IItemListView;
    FSelectedItemListView: IItemListView;
    FOrderedItemListView : IItemListView;

    procedure createItemListView;
    procedure createSelectedItemListView;
    procedure createOrderedItemListView;

    procedure showData;
    procedure subscribeOnDataChange;
  public
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

procedure TfrmMain.btnClearClick(Sender: TObject);
begin
  FItemListView.Obj.clearItemViews;
  FSelectedItemListView.Obj.clearItemViews;
  FOrderedItemListView.Obj.clearItemViews;
end;

procedure TfrmMain.btnLoadDataClick(Sender: TObject);
begin
  dmMain.loadData;
end;

procedure TfrmMain.btnShowDataClick(Sender: TObject);
begin
  showData;
end;

procedure TfrmMain.createItemListView;
begin
  FItemListView := TObj<TItemListView>.CreateInstance(TItemListView.Create(nil));
  FItemListView.Obj.Name := 'ItemListView';

  FItemListView.Obj.align := TAlign.alClient;
  FItemListView.Obj.Parent := tbItemList;
end;

procedure TfrmMain.createSelectedItemListView;
begin
  FSelectedItemListView := TObj<TItemListView>.CreateInstance(TItemListView.Create(self));
  FSelectedItemListView.Obj.Name := 'SelectedItemListView';

  FSelectedItemListView.Obj.align := TAlign.alClient;
  FSelectedItemListView.Obj.Parent := tbSelectedItemList;
end;

procedure TfrmMain.createOrderedItemListView;
begin
  FOrderedItemListView := TObj<TItemListView>.CreateInstance(TItemListView.Create(self));
  FOrderedItemListView.Obj.Name := 'OrderedItemListView';

  FOrderedItemListView.Obj.align := TAlign.alClient;
  FOrderedItemListView.Obj.Parent := tbOrderedItemList;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  pcPages.ActivePage := tbHome;

  createItemListView;
  createSelectedItemListView;
  createOrderedItemListView;
end;

procedure TfrmMain.FormShow(Sender: TObject);
begin
  subscribeOnDataChange;
  dmMain.LoadData;
end;

procedure TfrmMain.showData;
const
  MaxItemCount = 0;
begin
  begin
    var count := 0;
    FItemListView.Obj.bindData(dmMain.GetItemList(
        function (Item : TItem) : boolean
        begin
          inc(count);
          result := (MaxItemCount = 0) or (count < MaxItemCount);
        end)
    );
  end;

  begin
    var count := 0;
    FSelectedItemListView.Obj.bindData(dmMain.GetItemList(
        function (Item : TItem) : boolean
        begin
          if Item.ClickCount > 0 then
          begin
            inc(count);
            result := (MaxItemCount = 0) or (count < MaxItemCount);
          end
          else
            result := false;
        end)
    );
  end;

  begin
    var count := 0;
    var itemList := dmMain.GetItemList(
        function (Item : TItem) : boolean
        begin
          inc(count);
          result := (MaxItemCount = 0) or (count < MaxItemCount);
        end);

    itemList.Obj.Sort (
      TComparer<TItem>.Construct(
        function (const item1, item2 : TItem) : Integer
        begin
          var c := item2.ClickCount - item1.ClickCount;

          if c = 0 then
            result := CompareText(item1.Code, item2.Code)
          else
            result := c;
        end
      )
    );

    FOrderedItemListView.Obj.bindData(itemList);
  end;

end;

procedure TfrmMain.subscribeOnDataChange;
begin
  dmMain.OnLoadData := procedure
                        begin
                          // I need to use the timer to help avoiding access violation
                          // due to UI controls being referenced and deleted and recreate
                          // at the same time.
                          tmrDelayedShowData.Enabled := true;
                        end;
end;

procedure TfrmMain.tmrDelayedShowDataTimer(Sender: TObject);
begin
  tmrDelayedShowData.Enabled := false;
  showData;
end;

end.
