unit FrameItemList;

interface

uses
  AutoHeightFlowLayout, ModelUnit, Generics.Collections, IObj, MainDM,
  FrameItemDetails, EventHandlers, System.Threading,

  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.ExtCtrls;

type
  IItemView = IObj<TItemView>;
  TIItemViewList = TList<IItemView>;
  IIItemViewList = IObj<TIItemViewList>;

  TfrItemList = class(TFrame)
  private
    FLayoutItems : TAutoHeightFlowLayout;
    FItemList : IItemList;
    FItemViews : IIItemViewList;

    procedure createItemViews;
    procedure createLayoutItems;

    procedure ItemViewMouseEnter(Sender: TObject);
    procedure ItemViewMouseLeave(Sender: TObject);

    procedure ItemViewClick1(Sender: TObject);
    procedure ItemViewClick2(Sender: TObject);
  public
    constructor Create(AOwner: TComponent); override;

    procedure bindData(itemList : IItemList);
    procedure refreshData;

    procedure clearItemViews;
  end;

  TItemListView = TfrItemList;
  IItemListView = IObj<TItemListView>;

implementation

{$R *.dfm}

{ TfrItemList }

procedure TfrItemList.bindData(itemList: IItemList);
begin
  FLayoutItems.LockDrawing;

  FItemList := ItemList;
  clearItemViews;

  for var item in FItemList.Obj do
  begin
    var itemView := TObj<TItemView>.CreateInstance(TItemView.Create(nil));
    itemView.Obj.Name := 'Item' + item.Code;
    itemView.Obj.bindData(item);

    itemView.Obj.Margins.Top := 10;
    itemView.Obj.Margins.Bottom := 10;
    itemView.Obj.Margins.Left := 10;
    itemView.Obj.Margins.Right := 10;
    itemView.Obj.AlignWithMargins := true;

    itemView.Obj.parent := FLayoutItems;
    FItemViews.Obj.Add(itemView);

    ItemView.Obj.OnItemMouseEnter.setHandler(ItemViewMouseEnter);
    ItemView.Obj.OnItemMouseLeave.setHandler(ItemViewMouseLeave);

    ItemView.Obj.OnItemClick.addHandler(ItemViewClick1);
    ItemView.Obj.OnItemClick.addHandler(ItemViewClick2);

    ItemViewMouseLeave(ItemView.Obj);
  end;

  FLayoutItems.UnLockDrawing;
end;

procedure TfrItemList.refreshData;
begin
  bindData(FItemList);
end;

procedure TfrItemList.clearItemViews;
begin
  for var i := FItemViews.Obj.Count - 1 downto 0 do
    FItemViews.Obj[i].Obj.Parent := nil;

  FItemViews.Obj.Clear;
end;

constructor TfrItemList.Create(AOwner: TComponent);
begin
  inherited;

  createLayoutItems;
  createItemViews;
end;

procedure TfrItemList.createItemViews;
begin
  FItemViews := TObj<TIItemViewList>.CreateInstance(TIItemViewList.Create);
end;

procedure TfrItemList.createLayoutItems;
begin
  FLayoutItems := TAutoHeightFlowLayout.Create(self);
  FLayoutItems.Parent := self;
  FLayoutItems.Align := TAlign.alTop;
  FLayoutItems.Visible := true;
  FLayoutItems.Height := FLayoutItems.Height;

  FLayoutItems.Padding.Left := 20;
  FLayoutItems.Padding.Right := 20;
  FLayoutItems.Padding.Top := 20;
  FLayoutItems.Padding.Bottom := 20;

  FLayoutItems.BevelInner := bvNone;
  FLayoutItems.BevelOuter := bvNone;
end;

procedure TfrItemList.ItemViewClick1(Sender: TObject);
begin
  ShowMessage('Itemview clicked ' + TItemView(Sender).Item.Code);
end;

procedure TfrItemList.ItemViewClick2(Sender: TObject);
begin
  TItemView(Sender).Item.IncrementClickCount;
  TItemView(Sender).refreshData;
  dmMain.refreshData;
end;

procedure TfrItemList.ItemViewMouseEnter(Sender: TObject);
begin
  if Sender is TItemView then
  begin
    TItemView(Sender).setBGColor(clWebLightBlue);
    TItemView(Sender).setScales(1.1, 1.1);
  end;
end;

procedure TfrItemList.ItemViewMouseLeave(Sender: TObject);
begin
  if Sender is TItemView then
  begin
    TItemView(Sender).setBGColor(clLtGray);
    TItemView(Sender).setScales(1, 1);
  end;
end;

end.
