unit FrameItemDetails;

interface

uses
  ModelUnit, IObj, EventHandlers,

  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls;

type
  TfrItemDetails = class(TFrame)
    lblItem: TLabel;

    procedure lblItemMouseEnter(Sender: TObject);
    procedure lblItemMouseLeave(Sender: TObject);

    procedure lblItemClick(Sender: TObject);

  private
    FWidth, FHeight, FFontSize : Double;
    FItem : TItem;

    FOnItemMouseLeave: IObj<TMethodEvents>;
    FOnItemClick: IObj<TMethodEvents>;
    FOnItemMouseEnter: IObj<TMethodEvents>;

    procedure createOnItemClick;
    procedure createOnItemMouseEnter;
    procedure createOnItemMouseLeave;

    function getOnItemClick: TMethodEvents;
    function getOnItemMouseEnter: TMethodEvents;
    function getOnItemMouseLeave: TMethodEvents;
  public
    property Item : TItem read FItem;

    property OnItemClick : TMethodEvents read getOnItemClick;
    property OnItemMouseEnter : TMethodEvents read getOnItemMouseEnter;
    property OnItemMouseLeave : TMethodEvents read getOnItemMouseLeave;

    procedure bindData(item: TItem);
    procedure refreshData;

    procedure setScales(const scaleX, scaleY : Double);
    procedure setBGColor(const color : TColor);

    constructor Create(AOwner : TComponent); override;
  end;

  TItemView = TfrItemDetails;
  IItemView = IObj<TItemView>;

implementation

{$R *.dfm}

{ TfrItemDetails }

procedure TfrItemDetails.bindData(item: TItem);
begin
  FItem := item;
  lblItem.Caption := item.Description;
end;

constructor TfrItemDetails.Create(AOwner: TComponent);
begin
  inherited;
  FWidth := width;
  FHeight := height;
  FFontSize := lblItem.Font.Size;

  createOnItemClick;
  createOnItemMouseEnter;
  createOnItemMouseLeave;
end;

procedure TfrItemDetails.createOnItemClick;
begin
  FOnItemClick := TObj<TMethodEvents>.CreateInstance(TMethodEvents.Create(nil));
end;

procedure TfrItemDetails.createOnItemMouseEnter;
begin
  FOnItemMouseEnter := TObj<TMethodEvents>.CreateInstance(TMethodEvents.Create(nil));
end;

procedure TfrItemDetails.createOnItemMouseLeave;
begin
  FOnItemMouseLeave := TObj<TMethodEvents>.CreateInstance(TMethodEvents.Create(nil));
end;

function TfrItemDetails.getOnItemClick: TMethodEvents;
begin
  result := FOnItemClick.Obj;
end;

function TfrItemDetails.getOnItemMouseEnter: TMethodEvents;
begin
  result := FOnItemMouseEnter.Obj;
end;

function TfrItemDetails.getOnItemMouseLeave: TMethodEvents;
begin
  result := FOnItemMouseLeave.Obj;
end;

procedure TfrItemDetails.lblItemClick(Sender: TObject);
begin
  FOnItemClick.Obj.execute(self);
end;

procedure TfrItemDetails.lblItemMouseEnter(Sender: TObject);
begin
  FOnItemMouseEnter.Obj.execute(self);
end;

procedure TfrItemDetails.lblItemMouseLeave(Sender: TObject);
begin
  FOnItemMouseLeave.Obj.execute(self);
end;

procedure TfrItemDetails.refreshData;
begin
  bindData(FItem);
end;

procedure TfrItemDetails.setBGColor(const color: TColor);
begin
  lblItem.Color := color;
end;

procedure TfrItemDetails.setScales(const scaleX, scaleY: Double);
begin
  Width := round(scaleX * FWidth);
  Height := round(scaleY * FHeight);
  lblItem.Font.Size := round(FFontSize * scaleX);
end;

end.
