unit Unit1;

interface

uses
  IObj, RestMessageCenter,

  DBXJSON, JSON, Rest.Json, Datasnap.DSProxyRest, Datasnap.DSClientRest,
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs, FMX.Memo.Types,
  FMX.Edit, FMX.StdCtrls, FMX.Controls.Presentation, FMX.ScrollBox, FMX.Memo,
  Datasnap.DSCommon, Data.DBXDataSnap, Data.DBXCommon, IPPeerClient, Data.DB,
  Data.SqlExpr, FMX.ListBox, Data.DbxHTTPLayer,
  FMX.TabControl;

type
  TForm1 = class(TForm)
    btnBroadCast: TButton;
    edBroadCast: TEdit;
    edHostName: TEdit;
    edPort: TEdit;
    cbProtocol: TComboBox;
    radLocal: TRadioButton;
    edChannel: TEdit;
    TabControl1: TTabControl;
    TabItemBroadcast: TTabItem;
    TabItemConnection: TTabItem;
    btnConnect: TButton;
    lblConnectStatus: TLabel;
    Memo1: TMemo;
    procedure radLocalChange(Sender: TObject);
    procedure btnConnectClick(Sender: TObject);
    procedure btnBroadCastClick(Sender: TObject);
  private
    FDSRestConnection: IObj<TDSRestConnection>;
    FRestMessageCenter: IObj<TRestMessageCenter>;

    procedure createServerConnection();
    procedure connectToServer(const updateStatus : boolean);

    procedure createRestMessageCenter();

    function channel : string;
    function communicationProtocol : string;
    function hostName : string;
    function portNumber: integer;
    function connectStatus : string;

    procedure QueueLogMsg(const s : string);
  public
  end;

var
  Form1: TForm1;

implementation

{$R *.fmx}

procedure TForm1.btnBroadCastClick(Sender: TObject);
begin
  FRestMessageCenter.Obj.broadCast(channel, self.edBroadCast.Text);
end;

procedure TForm1.btnConnectClick(Sender: TObject);
begin
  connectToServer(true);
  createRestMessageCenter();

  FRestMessageCenter.Obj.CreateChannel(channel);
  FRestMessageCenter.Obj.OnMessage[channel].subscribe(self,
    procedure (s: string)
    begin
      QueueLogMsg(s);
    end
  );
end;

function TForm1.Channel: string;
begin
  result := edChannel.Text;
end;

function TForm1.communicationProtocol: string;
begin
  result := cbProtocol.Items[cbProtocol.ItemIndex];
end;

function TForm1.connectStatus: string;
begin
  var b := assigned(FDSRestConnection);
  if b then
  begin
    try
      FDSRestConnection.Obj.TestConnection([toNoLoginPrompt]);
    except
      b := false;
    end;
  end;

  if b then
    result := 'Yes'
  else
    result := 'No';
end;

procedure TForm1.connectToServer(const updateStatus : boolean);
begin
  if not assigned(FDSRestConnection) then
    createServerConnection;

  self.FDSRestConnection.Obj.Protocol := communicationProtocol;
  self.FDSRestConnection.Obj.Host := hostName;
  self.FDSRestConnection.Obj.Port := portNumber;

  if updateStatus then
    lblConnectStatus.Text := Format('Connected: %s', [ConnectStatus]);
end;

procedure TForm1.createRestMessageCenter;
begin
  FRestMessageCenter := TObj<TRestMessageCenter>.CreateInstance(TRestMessageCenter.Create(nil));
  FRestMessageCenter.Obj.Connection := self.FDSRestConnection.Obj;
end;

procedure TForm1.createServerConnection;
begin
  FDSRestConnection := TObj<TDSRestConnection>.CreateInstance(TDSRestConnection.Create(nil));
  FDSRestConnection.Obj.LoginPrompt := false;
end;

function TForm1.hostName: string;
begin
  result := edHostName.Text;
end;

function TForm1.portNumber: integer;
begin
  result := StrToIntDef(edPort.Text, 80);
end;

procedure TForm1.QueueLogMsg(const s: string);
begin
  TThread.Queue(nil,
    procedure
    begin
      memo1.Lines.Add(s);
    end
  );
end;

procedure TForm1.radLocalChange(Sender: TObject);
begin
  edHostName.Text := 'localhost';
  cbProtocol.ItemIndex := cbProtocol.Items.IndexOf('http');
  edPort.Text := '8080';
end;

end.
